#include-once
#include "Bass.au3"
#include "BASSASIOConstants.au3"
; #INDEX# =======================================================================================================================
; Title .........: BASSASIO.au3
; Description ...: Wrapper for BASSASIO.dll
; Author ........: Eukalyptus
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;			_BASS_ASIO_GetVersion
;			_BASS_ASIO_ErrorGetCode
;			_BASS_ASIO_GetDeviceInfo
;			_BASS_ASIO_SetDevice
;			_BASS_ASIO_GetDevice
;			_BASS_ASIO_Init
;			_BASS_ASIO_Free
;			_BASS_ASIO_ControlPanel
;			_BASS_ASIO_GetInfo
;			_BASS_ASIO_SetRate
;			_BASS_ASIO_GetRate
;			_BASS_ASIO_Start
;			_BASS_ASIO_Stop
;			_BASS_ASIO_IsStarted
;			_BASS_ASIO_GetLatency
;			_BASS_ASIO_GetCPU
;			_BASS_ASIO_Monitor
;			_BASS_ASIO_ChannelGetInfo
;			_BASS_ASIO_ChannelReset
;			_BASS_ASIO_ChannelEnable
;			_BASS_ASIO_ChannelEnableMirror
;			_BASS_ASIO_ChannelJoin
;			_BASS_ASIO_ChannelPause
;			_BASS_ASIO_ChannelIsActive
;			_BASS_ASIO_ChannelSetFormat
;			_BASS_ASIO_ChannelGetFormat
;			_BASS_ASIO_ChannelSetRate
;			_BASS_ASIO_ChannelGetRate
;			_BASS_ASIO_ChannelSetVolume
;			_BASS_ASIO_ChannelGetVolume
;			_BASS_ASIO_ChannelGetLevel
;			_BASS_ASIO_Startup
; ===============================================================================================================================
Global $_ghBassASIODll = -1
Global $BASS_ASIO_DLL_UDF_VER = "1.0.1.0"
Global $BASS_ASIO_UDF_VER = "10.0"

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_STARTUP
; Description ...: Starts up BassCD functions.
; Syntax ........: _BASS_ASIO_STARTUP($sBassASIODll = "")
; Parameters ....: -	$sBassASIODll	-	The relative path to BassAsio.dll.
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR
;									@error will be set to-
;										- $BASS_ERR_DLL_NO_EXIST	-	File could not be found.
; Author ........: Prog@ndy
; Modified ......: Eukalyptus
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_STARTUP($sBassASIODll = "")
	If $_ghBassASIODll <> -1 Then Return True
	If Not $sBassASIODll Then $sBassASIODll = @ScriptDir & "\bassasio.dll"

	If Not FileExists($sBassASIODll) Then Return SetError($BASS_ERR_DLL_NO_EXIST, 0, False)

	Local $sBit = __BASS_LibraryGetArch($sBassASIODll)
	Select
		Case $sBit = "32" And @AutoItX64
			ConsoleWrite(@CRLF & "!BassASIO.dll is for 32bit only!" & @CRLF & "Run/compile Script at 32bit" & @CRLF)
		Case $sBit = "64" And Not @AutoItX64
			ConsoleWrite(@CRLF & "!BassASIO.dll is for 64bit only!" & @CRLF & "use 32bit version of BassASIO.dll" & @CRLF)
	EndSelect

	If $BASS_STARTUP_VERSIONCHECK Then
		If Not @AutoItX64 And _VersionCompare(FileGetVersion($sBassASIODll), $BASS_ASIO_DLL_UDF_VER) <> 0 Then ConsoleWrite(@CRLF & "!This version of BASSASIO.au3 is made for BassASIO.dll V" & $BASS_ASIO_DLL_UDF_VER & ".  Please update" & @CRLF)
		If $BASS_ASIO_UDF_VER <> $BASS_UDF_VER Then ConsoleWrite("!This version of BASSASIO.au3 (v" & $BASS_ASIO_UDF_VER & ") may not be compatible to BASS.au3 (v" & $BASS_UDF_VER & ")" & @CRLF)
	EndIf

	$_ghBassASIODll = DllOpen($sBassASIODll)
	Return $_ghBassASIODll <> -1
EndFunc   ;==>_BASS_ASIO_STARTUP

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelEnable
; Description ...: Enables/disables processing of a channel.
; Syntax ........: _BASS_ASIO_ChannelEnable($input, $channel, $proc = 0, $user = 0)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
;					-	$user 			-	User instance data to pass to the callback function.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelEnable($input, $channel, $proc = 0, $user = 0)
	Local $proc_s = -1
	If IsString($proc) Then
		$proc_s = DllCallbackRegister($proc, "ptr", "dword;ptr;dword;ptr")
		$proc = DllCallbackGetPtr($proc_s)
	EndIf
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelEnable", "bool", $input, "dword", $channel, "ptr", $proc, "ptr", $user)
	If @error Then
		If $proc_s <> -1 Then DllCallbackFree($proc_s)
		Return SetError(1, 1, 0)
	EndIf
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then
		If $proc_s <> -1 Then DllCallbackFree($proc_s)
		Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	EndIf
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelEnable

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelEnableMirror
; Description ...: Enables an output channel, and makes it mirror another channel.
; Syntax ........: _BASS_ASIO_ChannelEnableMirror($channel, $input2, $channel2)
; Parameters ....: -	;					-	$channel 			-	The output channel number... 0 = first.
;					-	$input2 			-	Mirroring an input channel? FALSE = an output channel.
;					-	$channel2 			-	The channel to mirror.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelEnableMirror($channel, $input2, $channel2)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelEnableMirror", "dword", $channel, "bool", $input2, "dword", $channel2)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelEnableMirror

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelGetFormat
; Description ...: Retrieves a channel's sample format.
; Syntax ........: _BASS_ASIO_ChannelGetFormat($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns If an error occurs, -1 is returned, use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelGetFormat($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_ChannelGetFormat", "bool", $input, "dword", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelGetFormat

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelGetInfo
; Description ...: Retrieves information on a channel.
; Syntax ........: _BASS_ASIO_ChannelGetInfo($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns If successful, TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelGetInfo($input, $channel)
	Local $aRet[3]
	Local $BASS_asio_ret_struct = DllStructCreate($BASS_ASIO_CHANNELINFO)
	DllCall($_ghBassASIODll, "dword", "BASS_ASIO_ChannelGetInfo", "dword", $input, "dword", $channel, "ptr", DllStructGetPtr($BASS_asio_ret_struct))
	Local $error = _BASS_ASIO_ErrorGetCode()
	If $error <> 0 Then
		Return SetError($error, "", 0)
	Else
		For $i = 0 To 2
			$aRet[$i] = DllStructGetData($BASS_asio_ret_struct, $i + 1)
		Next
		Return SetError(0, "", $aRet)
	EndIf
EndFunc   ;==>_BASS_ASIO_ChannelGetInfo

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelGetLevel
; Description ...: Retrieves the level (peak amplitude) of a channel.
; Syntax ........: _BASS_ASIO_ChannelGetLevel($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns If an error occurs, -1 is returned, use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelGetLevel($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "float", "BASS_ASIO_ChannelGetLevel", "bool", $input, "dword", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelGetLevel

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelGetRate
; Description ...: Retrieves a channel's sample rate.
; Syntax ........: _BASS_ASIO_ChannelGetRate($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns If successful, the channel's sample rate is returned (0 = device rate), else -1 is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelGetRate($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "double", "BASS_ASIO_ChannelGetRate", "bool", $input, "dword", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelGetRate

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelGetVolume
; Description ...: Retrieves a channel's volume setting.
; Syntax ........: _BASS_ASIO_ChannelGetVolume($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first, -1 = master.
; Return values .: Success      - Returns If successful, the channel's volume setting is returned, else -1 is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelGetVolume($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "float", "BASS_ASIO_ChannelGetVolume", "bool", $input, "int", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelGetVolume

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelIsActive
; Description ...: Checks if a channel is enabled for processing.
; Syntax ........: _BASS_ASIO_ChannelIsActive($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns The return value is one of the following.
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelIsActive($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_ChannelIsActive", "bool", $input, "dword", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelIsActive

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelJoin
; Description ...: Join a channel to another.
; Syntax ........: _BASS_ASIO_ChannelJoin($input, $channel, $channel2)
; Parameters ....: -	;					-	$input 			-	Dealing with input channels? FALSE = output channels.
;					-	$channel 			-	The input/output channel number... 0 = first.
;					-	$channel2 			-	The channel to join it to... -1 = remove current join.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelJoin($input, $channel, $channel2)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelJoin", "bool", $input, "dword", $channel, "int", $channel2)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelJoin

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelPause
; Description ...: Suspends processing of a channel.
; Syntax ........: _BASS_ASIO_ChannelPause($input, $channel)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelPause($input, $channel)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelPause", "bool", $input, "dword", $channel)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelPause

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelReset
; Description ...: Resets the attributes of a channel (or all channels).
; Syntax ........: _BASS_ASIO_ChannelReset($input, $channel, $flags)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first, -1 = all channels.
;					-	$flags 			-	The attributes to reset. A combination of the following.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelReset($input, $channel, $flags)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelReset", "bool", $input, "int", $channel, "dword", $flags)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelReset

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelSetFormat
; Description ...: Sets a channel's sample format.
; Syntax ........: _BASS_ASIO_ChannelSetFormat($input, $channel, $format)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
;					-	$format 			-	The sample format. One of the following.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelSetFormat($input, $channel, $format)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelSetFormat", "bool", $input, "dword", $channel, "dword", $format)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelSetFormat

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelSetRate
; Description ...: Sets a channel's sample rate.
; Syntax ........: _BASS_ASIO_ChannelSetRate($input, $channel, $rate)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first.
;					-	$rate 			-	The sample rate... 0 = device rate.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelSetRate($input, $channel, $rate)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelSetRate", "bool", $input, "dword", $channel, "double", $rate)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelSetRate

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ChannelSetVolume
; Description ...: Sets a channel's volume.
; Syntax ........: _BASS_ASIO_ChannelSetVolume($input, $channel, $volume)
; Parameters ....: -	;					-	$input 			-	Dealing with an input channel? FALSE = an output channel.
;					-	$channel 			-	The input/output channel number... 0 = first, -1 = master.
;					-	$volume 			-	The volume level... 0 (silent) - 1.0 (normal). Above 1.0 amplifies the sound.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ChannelSetVolume($input, $channel, $volume)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ChannelSetVolume", "bool", $input, "int", $channel, "float", $volume)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ChannelSetVolume

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ControlPanel
; Description ...: Displays the driver's control panel.
; Syntax ........: _BASS_ASIO_ControlPanel()
; Parameters ....: -
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ControlPanel()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_ControlPanel")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ControlPanel


; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_ErrorGetCode
; Description ...: Retrieves the error code for the most recent BASSASIO function call in the current thread.
; Syntax ........: _BASS_ASIO_ErrorGetCode()
; Parameters ....: -
; Return values .: Success      - Returns If no error occurred during the last BASSASIO function call then BASS_OK is returned, else one of the BASS_ERROR values is returned. See the function description for an explanation of what the error code means.
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_ErrorGetCode()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_ErrorGetCode")
	If @error Then Return SetError(1, 1, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_ErrorGetCode

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_Free
; Description ...: Releases the device/driver.
; Syntax ........: _BASS_ASIO_Free()
; Parameters ....: -
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_Free()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_Free")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_Free

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetCPU
; Description ...: Retrieves the current CPU usage of BASSASIO.
; Syntax ........: _BASS_ASIO_GetCPU()
; Parameters ....: -
; Return values .: Success      - Returns The BASSASIO CPU usage as a percentage of total CPU time.
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetCPU()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "float", "BASS_ASIO_GetCPU")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_GetCPU

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetDevice
; Description ...: Retrieves the device setting in the current thread.
; Syntax ........: _BASS_ASIO_GetDevice()
; Parameters ....: -
; Return values .: Success      - Returns If successful, the device number is returned, else -1 is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetDevice()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "int", "BASS_ASIO_GetDevice")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_GetDevice

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetDeviceInfo
; Description ...: Retrieves information on a device.
; Syntax ........: _BASS_ASIO_GetDeviceInfo($device)
; Parameters ....: -	;					-	$device 			-	The device to get the information of... 0 = first.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetDeviceInfo($device)
	Local $aRet[3]
	Local $BASS_asio_ret_struct = DllStructCreate($BASS_ASIO_DEVICEINFO)
	Local $bass_asio_ret = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_GetDeviceInfo", "dword", $device, "ptr", DllStructGetPtr($BASS_asio_ret_struct))
	Local $error = _BASS_ASIO_ErrorGetCode()
	If $error <> 0 Then
		Return SetError($error, "", 0)
	Else
		$aRet[0] = $bass_asio_ret[0]
		$aRet[1] = _BASS_PtrStringRead(DllStructGetData($BASS_asio_ret_struct, 1))
		$aRet[2] = _BASS_PtrStringRead(DllStructGetData($BASS_asio_ret_struct, 2))
		Return SetError(0, "", $aRet)
	EndIf
EndFunc   ;==>_BASS_ASIO_GetDeviceInfo

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetInfo
; Description ...: Retrieves information on the device being used.
; Syntax ........: _BASS_ASIO_GetInfo()
; Parameters ....: -
; Return values .: Success      - Returns If successful, TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetInfo()
	Local $aRet[8]
	Local $BASS_asio_ret_struct = DllStructCreate($BASS_ASIO_INFO)
	DllCall($_ghBassASIODll, "dword", "BASS_ASIO_GetInfo", "ptr", DllStructGetPtr($BASS_asio_ret_struct))
	Local $error = _BASS_ASIO_ErrorGetCode()
	If $error <> 0 Then
		Return SetError($error, "", 0)
	Else
		For $i = 0 To 7
			$aRet[$i] = DllStructGetData($BASS_asio_ret_struct, $i + 1)
		Next
		Return SetError(0, "", $aRet)
	EndIf
EndFunc   ;==>_BASS_ASIO_GetInfo

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetLatency
; Description ...: Retrieves the latency of input or output channels.
; Syntax ........: _BASS_ASIO_GetLatency($input)
; Parameters ....: -	;					-	$input 			-	Get the input latency? Else, the output latency.
; Return values .: Success      - Returns If successful, the latency in samples is returned, else -1 is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetLatency($input)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_GetLatency", "bool", $input)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_GetLatency

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetRate
; Description ...: Retrieves the device's sample rate.
; Syntax ........: _BASS_ASIO_GetRate()
; Parameters ....: -
; Return values .: Success      - Returns If successful, the sample rate is returned, else -1 is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetRate()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "double", "BASS_ASIO_GetRate")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_GetRate

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_GetVersion
; Description ...: Retrieves the version of BASSASIO that is loaded.
; Syntax ........: _BASS_ASIO_GetVersion()
; Parameters ....: -
; Return values .: Success      - Returns The BASSASIO version. For example, 0x02040103 (hex), would be version 2.4.1.3
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_GetVersion()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "dword", "BASS_ASIO_GetVersion")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_GetVersion

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_Init
; Description ...: Initializes an ASIO device/driver.
; Syntax ........: _BASS_ASIO_Init($device)
; Parameters ....: -	;					-	$device 			-	The device to use... 0 = first device.
; Return values .: Success      - Returns If the device was successfully initialized, TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_Init($device)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_Init", "dword", $device)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_Init

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_IsStarted
; Description ...: Checks if processing has been started.
; Syntax ........: _BASS_ASIO_IsStarted()
; Parameters ....: -
; Return values .: Success      - Returns If the device has been started, then TRUE is returned, else FALSE is returned.
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_IsStarted()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_IsStarted")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_IsStarted

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_Monitor
; Description ...: Sets the direct input monitoring state.
; Syntax ........: _BASS_ASIO_Monitor($input, $output, $gain, $state, $pan)
; Parameters ....: -	;					-	$input 			-	The input to set the monitoring state of... -1 = all.
;					-	$output 			-	The suggested output for the monitoring.
;					-	$gain 			-	Suggested output gain... ranging 0 (silent) - 0x7FFFFFFF (+12dB), 0x20000000 = 0dB.
;					-	$state 			-	Monitoring state... 0 = off, 1 = on. Other settings may be possible depending on the drivers.
;					-	$pan 			-	Suggested output pan... 0 = left, 0x7FFFFFFF = right.
; Return values .: Success      - Returns If successful, TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_Monitor($input, $output, $gain, $state, $pan)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_Monitor", "int", $input, "dword", $output, "dword", $gain, "dword", $state, "dword", $pan)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_Monitor

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_SetDevice
; Description ...: Sets the device to use for subsequent calls in the current thread.
; Syntax ........: _BASS_ASIO_SetDevice($device)
; Parameters ....: -	;					-	$device 			-	The device to use.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_SetDevice($device)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_SetDevice", "dword", $device)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_SetDevice

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_SetNotify
; Description ...: Sets a notification callback on the ASIO driver.
; Syntax ........: _BASS_ASIO_SetNotify($user)
; Parameters ....: -	;					-	$user 			-	User instance data to pass to the callback function.
; Return values .: Success      - Returns If successful, TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_SetNotify($user)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_SetNotify", "ptr", $user)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_SetNotify

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_SetRate
; Description ...: Sets the device's sample rate.
; Syntax ........: _BASS_ASIO_SetRate($rate)
; Parameters ....: -	;					-	$rate 			-	The sample rate.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_SetRate($rate)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_SetRate", "double", $rate)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_SetRate

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_Start
; Description ...: Starts the device.
; Syntax ........: _BASS_ASIO_Start($buflen)
; Parameters ....: -	;					-	$buflen 			-	Buffer length in samples... 0 = use current length.
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_Start($buflen)
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_Start", "dword", $buflen)
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_Start

; #FUNCTION# ====================================================================================================================
; Name ..........: _BASS_ASIO_Stop
; Description ...: Stops the device.
; Syntax ........: _BASS_ASIO_Stop()
; Parameters ....: -
; Return values .: Success      - Returns If successful, then TRUE is returned, else FALSE is returned. Use
;                  Failure      - Returns 0 and sets @ERROR to error returned by _BASS_ErrorGetCode()
; Author ........: Eukalyptus
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _BASS_ASIO_Stop()
	Local $BASS_ret_ = DllCall($_ghBassASIODll, "bool", "BASS_ASIO_Stop")
	If @error Then Return SetError(1, 1, 0)
	If $BASS_ret_[0] = $BASS_DWORD_ERR Then Return SetError(_BASS_ASIO_ErrorGetCode(), 0, 0)
	Return $BASS_ret_[0]
EndFunc   ;==>_BASS_ASIO_Stop