#include-once
#include "Bass.au3"
; #INDEX# =======================================================================================================================
; Title .........: BassTags.au3
; Description ...: Allows easy retrieval of tags using the handle of the currently playing stream.
; Author ........: Brett Francis (BrettF)
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;			_Bass_Tags_Startup()
;			_Bass_Tags_Read()
;			_Bass_Tags_GetLastErrorDesc()
;			_Bass_Tags_GetVersion()
; ===============================================================================================================================

Global $_ghBassTagsDll = -1
Global $BASS_TAGS_DLL_UDF_VER = "0.0.0.15"
Global $BASS_TAGS_UDF_VER = "10.0"
;Global $BASS_ERR_DLL_NO_EXIST = -1

; #FUNCTION# ====================================================================================================================
; Name ..........: _Bass_Tags_Startup
; Description ...: Starts up BASS functions.
; Syntax ........: _Bass_Tags_Startup($sBassTagsDll = "")
; Parameters ....: -	$sBassTagsDll	-	The relative path to BassTags.dll.
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR
;									@error will be set to-
;										- $BASS_ERR_DLL_NO_EXIST	-	File could not be found.
; Author ........: Prog@ndy
; Modified ......: Eukalyptus
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Bass_Tags_Startup($sBassTagsDll = "")
	If $_ghBassTagsDll <> -1 Then Return True
	If Not $sBassTagsDll Then $sBassTagsDll = @ScriptDir & "\basstags.dll"

	If Not FileExists($sBassTagsDll) Then Return SetError($BASS_ERR_DLL_NO_EXIST, 0, False)

	Local $sBit = __BASS_LibraryGetArch($sBassTagsDll)
	Select
		Case $sBit = "32" And @AutoItX64
			ConsoleWrite(@CRLF & "!BassTags.dll is for 32bit only!" & @CRLF & "Run/compile Script at 32bit" & @CRLF)
		Case $sBit = "64" And Not @AutoItX64
			ConsoleWrite(@CRLF & "!BassTags.dll is for 64bit only!" & @CRLF & "use 32bit version of BassTags.dll" & @CRLF)
	EndSelect

	If $BASS_STARTUP_VERSIONCHECK Then
		If Not @AutoItX64 And _VersionCompare(FileGetVersion($sBassTagsDll), $BASS_TAGS_DLL_UDF_VER) <> 0 Then ConsoleWrite(@CRLF & "!This version of BASSTAGS.au3 is made for BassTAGS.dll V" & $BASS_TAGS_DLL_UDF_VER & ".  Please update" & @CRLF)
		If $BASS_TAGS_UDF_VER <> $BASS_UDF_VER Then ConsoleWrite("!This version of BASSTAGS.au3 (v" & $BASS_TAGS_UDF_VER & ") may not be compatible to BASS.au3 (v" & $BASS_UDF_VER & ")" & @CRLF)
	EndIf

	$_ghBassTagsDll = DllOpen($sBassTagsDll)
	Return $_ghBassTagsDll <> -1
EndFunc   ;==>_Bass_Tags_Startup

; #FUNCTION# ====================================================================================================================
; Name ..........: _Bass_Tags_Read
; Description ...:
; Syntax ........: _Bass_Tags_Read($hHandle, $sFMT)
; Parameters ....: -	$	-
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR as set by _Bass_ErrorGetCode()
;									@error will be set to-
;										- $	-
; Author ........: Brett Francis (BrettF)
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Bass_Tags_Read($hHandle, $sFMT)
	$_bTags_Ret_ = DllCall($_ghBassTagsDll, "str", "TAGS_Read", "dword", $hHandle, "str", $sFMT)
	If @error Then Return SetError(1, @error, 0)
	Return $_bTags_Ret_[0]
EndFunc   ;==>_Bass_Tags_Read

; #FUNCTION# ====================================================================================================================
; Name ..........: _Bass_Tags_GetLastErrorDesc
; Description ...:
; Syntax ........: _Bass_Tags_GetLastErrorDesc()
; Parameters ....: -	$	-
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR as set by _Bass_ErrorGetCode()
;									@error will be set to-
;										- $	-
; Author ........: Brett Francis (BrettF)
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Bass_Tags_GetLastErrorDesc()
	$_bTags_Ret_ = DllCall($_ghBassTagsDll, "str", "TAGS_GetLastErrorDesc")
	If @error Then Return SetError(1, @error, 0)
	Return $_bTags_Ret_[0]
EndFunc   ;==>_Bass_Tags_GetLastErrorDesc

; #FUNCTION# ====================================================================================================================
; Name ..........: _Bass_Tags_GetVersion
; Description ...:
; Syntax ........: _Bass_Tags_GetVersion()
; Parameters ....: -	$	-
; Return values .: Success      - Returns True
;                  Failure      - Returns False and sets @ERROR as set by _Bass_ErrorGetCode()
;									@error will be set to-
;										- $	-
; Author ........: Brett Francis (BrettF)
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:
; ===============================================================================================================================
Func _Bass_Tags_GetVersion()
	$_bTags_Ret_ = DllCall($_ghBassTagsDll, "dword", "TAGS_GetVersion")
	If @error Then Return SetError(1, @error, 0)
	Return $_bTags_Ret_[0]
EndFunc   ;==>_Bass_Tags_GetVersion