#Region Includes
#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>
#include <Color.au3>
#include <GDIPlus.au3>
#include <Misc.au3>
#include "BASS.au3"
#EndRegion


#Region Declarations
;~ Bass UDF
Global Const $bass_dll = _BASS_STARTUP(@ScriptDir & "\BASS.dll")

;~ Window
Global Const $wWidth = 1000
Global Const $wHeight = 1000
Global Const $wCenter_x = $wWidth / 2
Global Const $wCenter_y = $wHeight / 2
Global Const $center_x = @DesktopWidth/2
Global Const $center_y = @DesktopHeight/2
Global Const $wZero_x = $center_x - ($wWidth/2)
Global Const $wZero_y = $center_y - ($wHeight/2)

;~ Environment
Global Const $pi = 3.141592653589793
Global $scale = 0.05
Global $curr_scale
$curr_scale = $scale
Global $RightChLvl, $LeftChLvl
Global $trigger, $count, $hist_on, $audio_on, $zoom_on, $tooltip
$hist_on = -1
$audio_on = 1
$zoom_on = -1
$tooltip = 1

;~ Object
Global $mouseX = MouseGetPos(0) - $wZero_x
Global $mouseY = MouseGetPos(1) - $wZero_y
Global $radius = 150
Global $sradius = 10
Global $num = 90
Global $aPoints[1000][2]
$aPoints[0][0] = $num
Global $vibration_x = 0
Global $vibration_y = 0
Global $speed_x = 0.025
Global $speed_y = $speed_x
Global $hBrush_col[3]
$hBrush_col[0] = 0xFF
$hBrush_col[1] = 0x56
$hBrush_col[2] = 0xBB

;~ Possibility to read data from history for further programming (not used yet)
Global $hist[1000]
For $i = 0 To 999
   $hist[$i] = 0
Next
#EndRegion


#Region HotKeySet
HotKeySet("{UP}", "_num_up")
HotKeySet("{DOWN}", "_num_down")
HotKeySet("{h}", "_hist_on")
HotKeySet("{a}", "_audio_on")
HotKeySet("{z}", "_zoom_on")
HotKeySet("{c}", "_clear")
HotKeySet("{t}", "_tooltip")
#EndRegion


#Region ### Bass UDF
$file = FileOpenDialog("Open...", "", "MP3 Files (*.mp3)");
If @error Then
    MsgBox(0, "Error", "Could not load audio file" & @CR & "Error = " & @error)
    Exit
EndIf
_BASS_Init($BASS_DEVICE_CPSPEAKERS)
$MusicHandle = _BASS_StreamCreateFile(False, $file, 0, 0, 0)
_BASS_ChannelPlay($MusicHandle, 1)
$song_length = _BASS_ChannelGetLength($MusicHandle, $BASS_POS_BYTE)
#EndRegion ### Bass UDF


#Region Gui
$hGui = GUICreate("Form1", $wWidth, $wHeight, $wZero_x, $wZero_y, -2147483648)
GUISetState(@SW_SHOW)
Opt("GUIOnEventMode", 1)
GUISetOnEvent($GUI_EVENT_CLOSE, "SpecialEvents")
GUIRegisterMsg($WM_MOUSEWHEEL, "_WM_MOUSEWHEEL")
#EndRegion


#Region ### START GDI+ ###
_GDIPlus_Startup()

;~ Graphics
$hGraphic = _GDIPlus_GraphicsCreateFromHWND($hGui)

;~ Bitmaps
$hGraphicBitmap = _GDIPlus_BitmapCreateFromGraphics($wWidth, $wHeight, $hGraphic)
$hHistoryBitmap = _GDIPlus_BitmapCreateFromScan0($wWidth, $wHeight)
$hCurrentBitmap = _GDIPlus_BitmapCreateFromScan0($wWidth, $wHeight)

;~ Buffers
$hGraphicBuffer = _GDIPlus_ImageGetGraphicsContext($hGraphicBitmap)
$hHistoryBuffer = _GDIPlus_ImageGetGraphicsContext($hHistoryBitmap)
$hCurrentBuffer = _GDIPlus_ImageGetGraphicsContext($hCurrentBitmap)

;~ PixxelOfset
_GDIPlus_GraphicsSetPixelOffsetMode($hGraphicBuffer, $GDIP_PIXELOFFSETMODE_HIGHQUALITY)
_GDIPlus_GraphicsSetPixelOffsetMode($hHistoryBuffer, $GDIP_PIXELOFFSETMODE_HIGHQUALITY)
_GDIPlus_GraphicsSetPixelOffsetMode($hCurrentBuffer, $GDIP_PIXELOFFSETMODE_HIGHQUALITY)

;~ Smoothing
_GDIPlus_GraphicsSetSmoothingMode($hGraphicBuffer, 4)
_GDIPlus_GraphicsSetSmoothingMode($hHistoryBuffer, 4)
_GDIPlus_GraphicsSetSmoothingMode($hCurrentBuffer, 4)

;~ First Clearing
_clear()

;~ Pens
$hPen = _GDIPlus_PenCreate(0x7700000F, 1)

;~ Brushes
$hBrush = _GDIPlus_BrushCreateSolid(0xFF0000FF)

;~ Zoom
$hBuffer = _GDIPlus_BitmapCreateFromScan0($wWidth, $wHeight)
$hContext = _GDIPlus_ImageGetGraphicsContext($hBuffer)
_GDIPlus_GraphicsTranslateTransform($hContext, -($wWidth/2) * $scale, -($wHeight/2) * $scale)
_GDIPlus_GraphicsScaleTransform($hContext, 1 + $scale, 1 + $scale)
_GDIPlus_GraphicsSetInterpolationMode($hContext, 1)
#EndRegion ### END GDI+ ###


#Region Loop
MouseMove($center_x, $center_y)
While 1
   _Calc()
   _Color()
   If $zoom_on == 1 Then
	  _Draw_zoomer()
   Else
	  _Draw_normal()
   EndIf
   If $tooltip == 1 Then
	  ToolTip("Num=" & $num & @CRLF & "Hist=" & $hist_on & @CRLF & "Aud=" & $audio_on & @CRLF & "Zoom=" & $zoom_on & @CRLF & "ZoomSpeed=" & $curr_scale)
   EndIf
WEnd
#EndRegion


#Region Functions Process
Func _Calc()
;~    Get sound input
   $levels = _BASS_ChannelGetLevel($MusicHandle)
   $LeftChLvl = Round(_HiWord($levels) / $wHeight * 0.5, 0)
   $RightChLvl = Round(_LoWord($levels) / $wHeight * 0.5, 0)

;~ 	  Trigger for playin around with colours (in process)
   If $RightChLvl > 20 Then
	  $trigger = 1
	  $count += 1
   Else
	  $trigger = 0
   EndIf

;~    Add to history (not used yet)
   $hist[0] = $RightChLvl
   For $i = 2 To 1000
	  $hist[1001-$i] = $hist[1000-$i]
   Next

;~    Mouse position for outer radius and spinning radius
   $mouseX = MouseGetPos(0) - $wZero_x
   $mouseY = MouseGetPos(1) - $wZero_y

;~ 	  This makes the dots circle
   $vibration_x += $speed_x
   $vibration_y += $speed_y

;~ 	  Calculate the dots positions
   For $i = 1 To $num Step 1
	  $aPoints[$i][0] = $wCenter_x + ($radius+($center_y - $mouseY)*0.5) * sin(($i/$num)*2*$pi) + (sin($vibration_x+$i)*($sradius+($center_x - $mouseX)*0.1))
	  If $audio_on == 1 And Mod($i, 2) Then
		 $aPoints[$i][1] = $wCenter_y + ($radius+($center_y - $mouseY)*0.5) * cos(($i/$num)*2*$pi) + (cos($vibration_x+$i)*($sradius+($center_x - $mouseX)*0.1)) + $RightChLvl
	  ElseIf $audio_on == 1 And Not Mod($i, 2) Then
		 $aPoints[$i][1] = $wCenter_y + ($radius+($center_y - $mouseY)*0.5) * cos(($i/$num)*2*$pi) + (cos($vibration_x+$i)*($sradius+($center_x - $mouseX)*0.1)) - $RightChLvl
	  ElseIf $audio_on == -1 And Mod($i, 2) Then
		 $aPoints[$i][1] = $wCenter_y + ($radius+($center_y - $mouseY)*0.5) * cos(($i/$num)*2*$pi) + (cos($vibration_x+$i)*($sradius+($center_x - $mouseX)*0.1))
	  Else
		 $aPoints[$i][1] = $wCenter_y + ($radius+($center_y - $mouseY)*0.5) * cos(($i/$num)*2*$pi) + (cos($vibration_x+$i)*($sradius+($center_x - $mouseX)*0.1))
	  EndIf
   Next
EndFunc


Func _Color()
   For $i = 0 To 2
	  If $hBrush_col[$i] > 0x00 Then
		 $hBrush_col[$i] -= Random(0, 0x02, 1)
	  Else
		 $hBrush_col[$i] = 0xFF
	  EndIf
	  If $trigger == 1 And $count > 2 Then
		 $hBrush_col[$i] = 0xFF
		 $count = 0
	  EndIf
   Next
   _GDIPlus_BrushSetSolidColor($hBrush, "0xFF" & Hex(_ColorSetRGB($hBrush_col), 6))
EndFunc


Func _Draw_normal()
   ;~    Clear
   _GDIPlus_GraphicsDrawImageRect($hCurrentBuffer, $hHistoryBitmap, 0, 0, $wWidth, $wHeight)

   ;~    Draw pieces --> $hCurrentBuffer
   For $i = 1 To $num Step 1
	  _GDIPlus_GraphicsFillEllipse($hCurrentBuffer, $aPoints[$i][0]-4, $aPoints[$i][1]-4, 8, 8, $hBrush)
   Next
   _GDIPlus_GraphicsDrawPolygon($hCurrentBuffer, $aPoints, $hPen)

   ;~    Draw History --> $hHistoryBuffer
   If $hist_on == 1 Then
	  For $i = 1 To $num Step 1
		 _GDIPlus_GraphicsFillEllipse($hHistoryBuffer, $aPoints[$i][0]-4, $aPoints[$i][1]-4, 8, 8, $hBrush)
	  Next
	  _GDIPlus_GraphicsDrawPolygon($hHistoryBuffer, $aPoints, $hPen)
   EndIf

   ;~    Draw finally --> $hGraphicBuffer
   _GDIPlus_GraphicsDrawImageRect($hGraphicBuffer, $hHistoryBitmap,0, 0, $wWidth, $wHeight)
   _GDIPlus_GraphicsDrawImageRect($hGraphicBuffer, $hCurrentBitmap,0, 0, $wWidth, $wHeight)
   _GDIPlus_GraphicsDrawImageRect($hGraphic, $hGraphicBitmap, 0, 0, $wWidth, $wHeight)

;~    For switching between normal and zoomer, also draw to the $hContext
   _GDIPlus_GraphicsDrawImageRect($hContext, $hGraphicBitmap, 0, 0, $wWidth, $wHeight)
EndFunc


Func _Draw_zoomer()
;~ 		 Clear
   _GDIPlus_GraphicsDrawImageRect($hGraphicBuffer, $hBuffer, 0, 0, $wWidth, $wHeight)

   ;~    Draw pieces
   For $i = 1 To $num Step 1
	  _GDIPlus_GraphicsFillEllipse($hGraphicBuffer, $aPoints[$i][0]-4, $aPoints[$i][1]-4, 8, 8, $hBrush)
   Next
   _GDIPlus_GraphicsDrawPolygon($hGraphicBuffer, $aPoints, $hPen)

;~ 		 Draw finally
   _GDIPlus_GraphicsDrawImageRect($hGraphic, $hGraphicBitmap, 0, 0, $wWidth, $wHeight)
   _GDIPlus_GraphicsDrawImageRect($hContext, $hGraphicBitmap, 0, 0, $wWidth, $wHeight)
EndFunc


Func _Exit()
   _BASS_Stop()
   _BASS_Free()
   DllClose(@ScriptDir & "\BASS.dll")
   _GDIPlus_GraphicsDispose($hGraphic)
   _GDIPlus_PenDispose($hPen)
   _GDIPlus_BrushDispose($hBrush)
   _GDIPlus_Shutdown()
   Exit
EndFunc
#EndRegion


#Region Event Functions and HotKeySet
Func _WM_MOUSEWHEEL($hWnd, $iMsg, $wParam, $lParam)
   If $wParam = 0x00780000 Then
	  $scale = 0.0025
   ElseIf $wParam = 0xFF880000 Then
	  $scale = -0.0025
   EndIf
   $curr_scale += $scale
   _GDIPlus_GraphicsTranslateTransform($hContext, -($wWidth/2) * $scale, -($wHeight/2) * $scale)
   _GDIPlus_GraphicsScaleTransform($hContext, 1 + $scale, 1 + $scale)
   Return $GUI_RUNDEFMSG
EndFunc


Func _num_up()
   $num += 1
   $aPoints[0][0] = $num
EndFunc


Func _num_down()
   $num -= 1
   $aPoints[0][0] = $num
EndFunc


Func _hist_on()
   $hist_on *= -1
EndFunc


Func _audio_on()
   $audio_on *= -1
EndFunc


Func _tooltip()
   $tooltip *= -1
   ToolTip("")
EndFunc


Func _zoom_on()
   $zoom_on *= -1
   _GDIPlus_GraphicsDrawImageRect($hHistoryBuffer, $hGraphicBitmap, 0, 0, $wWidth, $wHeight)
EndFunc


Func _clear()
   _GDIPlus_GraphicsClear($hGraphicBuffer, 0xFFFFFFFF)
   _GDIPlus_GraphicsClear($hHistoryBuffer, 0xFFFFFFFF)
   _GDIPlus_GraphicsClear($hCurrentBuffer, 0xFFFFFFFF)
EndFunc


Func SpecialEvents()
   Select
      Case @GUI_CtrlId = $GUI_EVENT_CLOSE
         GUIDelete()
         _Exit()
   EndSelect
EndFunc
#EndRegion